/*
	Extended services for IRCb
	--------------------------

        For russian channels:
	First, you must type "!CP NNNN" (where NNNN - codepage number) from
	your IRC-client.

	New commands with this script (type it on channel via your IRC-client):

        !cp copepage_number	set russian codepage for channel (878,1251,866)
        !help			sending different information
	news add Text_of_news	add news (op only)
        news			read all news
	!idiot Text		translate for russian: "jib,cz" -> "訡" ;)

							Digi, 2006
*/

parse arg IRCdInstant
if IRCdInstant='' | \DATATYPE(IRCdInstant,'N') then IRCdInstant = 0

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

EvPipeName = '\PIPE\IRCb' || IRCdInstant || '\Events'
EvPipeState = psDisconnected
ReqPipeName = '\PIPE\IRCb' || IRCdInstant || '\Requests'

parse SOURCE sCmd sFunc SrcPath
SrcPath = FileSpec("drive",SrcPath) || FileSpec("path",SrcPath)
DataPath = SrcPath || 'data\'
Globals = 'DataPath Event. ReqPipeName result. Chan. CID cp. ChanCP weather.'

/* Max. news items */
MaxNewsLines = 10

/* Russian codepages */
cp.eng  = '`qwertyuiop[]asdfghjkl;''zxcvbnm,.~QWERTYUIOP{}ASDFGHJKL:"ZXCVBNM<>'
cp.878  = ''
cp.1251 = ''
cp.866  = '㪥뢠஫ᬨ'

ChanUserFlags = 'vhoaq'
cuf.Voice    = 1
cuf.Halfop   = 2
cuf.Operator = 3
cuf.Admin    = 4
cuf.Founder  = 5

/* Read installed codepages for channels */

FName = DataPath || 'cp.set'
i = 0
do while lines(FName)\=0
  line = linein(FName)
  parse upper var line ChanName' 'CPSet
  if CPSet='' then iterate
  i = i+1
  Chan.i.name = ChanName
  Chan.i.cp   = CPSet
end
call stream FName,'c','close'
Chan.0 = i


/* --- Main --- */

do forever

  /* Read new event */
  call WaitEvent
  say '--- New event received ---'
  say 'Bot: 'Event.Bot
  say 'Event: 'Event.Id
  say 'Sender: 'Event.Sender
  say 'Channel: 'Event.Channel
  say 'User: 'Event.User
  say 'Data: 'Event.Data

  if Event.Id='PubMsg' then  /* public message */
  do
    parse var Event.Sender Nick'!'trash /* cut NickName */
    parse var Event.Data Cmd Param      /* split message: "command param." */

    ChanName = translate(Event.Channel)
    call GetChan			/* Get channel's codepage (ChanCP)
					   and convert Cmd to lat. characters */
    Cmd = translate(Cmd)
    Param = space(Param)

    select
    when Cmd = '!HELP' | Cmd = '!GJVJOM' then 
      if Param = '' | left(Param,1)='~' then
      do
        /* send list of topics */

        line = "Read topics: '!help <topic>'. Available topics:"
        rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' notice "'line'"')
        if left(Param,1)='~' then TopicMask = substr(Param,2)
          else TopicMask = '*'
        rc = SysFileTree(DataPath''TopicMask'.txt', 'list', 'O', '**-*-')
        do i = 1 to list.0
          line = linein(list.i)
          call stream list.i,'c','close'
          if left(line,1)='.' then 
            line = ' - ' || translate(substr(line,2), value('cp.'ChanCP), cp.866)
          else line = ''
          parse value FileSpec("name",list.i) with topic '.' rc
          line = topic || line
          rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' notice "'line'"')
        end
      end
      else
        call SendTopicFile Param'.txt','notice'    /* send text */
    when Cmd = 'NEWS' then 
      do
        Cmd = translate(word(Param,1))
        if Cmd = 'ADD' then 
        do
          /* Add news item */

          i = WordIndex(Param,2)
          if IsSenderOper() & i>0 then
            call AddChanNews Event.Channel, Event.Sender, SubStr(Param, i)
        end
        else
          call SendChanNews Event.Channel	/* send nes items */
      end
    when Cmd = '!CP' then 
      if Param='?' then				/* received "!CP ?" */
      do
        if CID\=-1 then
          line = 'Current codepage for 'Event.Channel': 'Chan.CID.cp
        else line = 'No codepage installed for 'Event.Channel
        rc = SendRequest('SENDMSG 'Event.Bot' 'Event.Channel' notice "'line'"') 
      end
      else if IsSenderOper() then		/* set codepage "!CP NNNN" */
      do
        Param = translate(Param)
        if Param='' then Param='ENG' /* no arg. - reset to default codepage */
        if symbol('cp.'Param) \= 'VAR' then
          line = "Invalid parameter"
        else do
          if CID = -1 then	/* cp was not be instaled ==> new record */
          do
            CID = Chan.0+1
            Chan.0 = CID
            Chan.CID.name = ChanName
          end
          Chan.CID.cp = Param

          /* store changes */

          FName = DataPath || 'cp.set'
          call SysFileDelete FName
          do i = 1 to Chan.0
            if Chan.i.cp\='ENG' then
            call lineout FName, Chan.i.name' 'Chan.i.cp
          end
          call stream FName,'c','close'   
          line = 'Codepage for 'Event.Channel' installed'
        end
        rc = SendRequest('SENDMSG 'Event.Bot' 'Event.Channel' notice "'line'"') 
      end 
    when Cmd = '!IDIOT' then 
      if CID\=-1 then do
        line = 'idiot sayed: 'translate(Param ,value('cp.'ChanCP), cp.eng)
        rc = SendRequest('SENDMSG 'Event.Bot' 'Event.Channel' message "'line'"')
      end
    OTHERWISE
    end
  end
end


/*----------------------------------------------*/
/*|                PIPE interface              |*/
/*----------------------------------------------*/

WaitEvent: procedure expose EvPipeState EvPipeName Event.
  do forever
    if EvPipeState = psDisconnected then
    do
      rc = stream(EvPipeName,'C','OPEN READ')
      parse var rc rc':'N
      if rc='READY' then EvPipeState = psConnected
      else call SysSleep .5
    end
    else do
      call SysSleep 0.01
      Line = charin(EvPipeName,,1024)
      if Line = '' then 
      do
        EvPipeState = psDisconnected
        rc = stream(EvPipeName,'C','CLOSE')
      end
      else do
        parse var Line Event.Bot' 'Event.Id' 'Event.Sender' 'Event.Channel' 'Event.User' 'Event.Data
        return
      end
    end
  end
return

SendRequest: procedure expose(Globals)
  rc = stream(ReqPipeName,'C','OPEN')
  Parse value rc with PipeState ':' OS2RC
  if OS2RC=3 then return ''
  if OS2RC=231 then 
    rc = SysWaitNamedPipe(ReqPipeName,2000)
  rc = charout(ReqPipeName,Arg(1))
  Cnt = 0
  do forever
    line = charin(ReqPipeName,,1024)
    if line = '' | left(line,1) = '/' then leave 
    Cnt = Cnt+1
    result.Cnt = line
  end
  result.0 = Cnt
  rc = stream(ReqPipeName,'C','CLOSE')
return substr(line,2)

/*----------------------------------------------*/
/*|              Service functions             |*/
/*----------------------------------------------*/

ChanToFName:
  line = strip(arg(1))
  ChanNameValidChr = XRANGE('a','z') || XRANGE('A','Z') || XRANGE('0','9') || '!#$^()-_+~;.,[]{}'
  fname = ''
  do forever
    rc = verify(line,ChanNameValidChr,'N')
    if rc\=0 then
    do
      fname = fname || left(line,rc-1)
      fname = fname || '%' || C2X(substr(line,rc,1))
      line = substr(line,rc+1)
      pos = rc+1
    end
    else leave
  end
  fname = fname || line
return fname

SendTopicFile: procedure expose(Globals)
  Param = arg(1)
  SendType = arg(2)

  TextFile = DataPath || Param
  if stream(TextFile,'c','QUERY EXISTS') = '' then
    rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' 'SendType' "Text is not available"')
  else do
    rc = ''
    do while lines(TextFile)\=0
      line = translate(linein(TextFile), value('cp.'ChanCP), cp.866)
      if rc = '' & left(line,1)='.' then iterate
      rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' 'SendType' "'line'"')
    end
    call stream TextFile,'c','close'
  end
return

AddChanNews: procedure expose(Globals) expose MaxNewsLines
  /* New item - first line of file */

  FName  = DataPath || ChanToFName(arg(1)) || '.news'
  Sender = arg(2)
  Text   = translate(arg(3),'`','"')

  do i=1 by 1 while lines(FName)\=0
    line.i = linein(FName)
  end
  call stream FName,'c','close'
  if i>MaxNewsLines then i = MaxNewsLines
  line.0 = i-1
  call SysFileDelete FName
  NewLine = DATE('E')' 'Sender' 'Text
  rc = lineout(FName, NewLine)
  do i=1 to line.0
    rc = lineout(FName, line.i)
  end
  call stream FName,'c','close'
  rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Channel' notice "News stored"')
return

SendChanNews: procedure expose(Globals)
  FName  = DataPath || ChanToFName(arg(1)) || '.news'
  From = arg(2)

  if stream(FName,'c','QUERY EXISTS') = '' then
    rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' notice "No news on the channel"')
  else do
    rc = ''
    do while lines(FName)\=0
      parse value linein(FName) with rData' 'rNick'!'rIdent' 'Text
      line = rData' 'rNick': 'Text
      rc = SendRequest('STREAMSTR 'Event.Bot' 'Event.Sender' notice "'line'"')
    end
    call stream FName,'c','close'
  end
return

GetChan:
  ChanCP = 'eng'
  CID = -1
  do i = 1 to Chan.0
    if Chan.i.name = ChanName then
    do
      ChanCP = Chan.i.cp
      Cmd = translate(Cmd, cp.eng, value('cp.'ChanCP))
      CID = i
      leave
    end
  end
return

IsSenderOper: procedure expose(Globals) expose cuf. ChanUserFlags
  if SendRequest('CHANNICKINFO 'Event.Bot' 'Event.Channel' 'Event.Sender)='Ok' then
  do
    parse var result.1 User' 'Flag
    if pos(Flag,ChanUserFlags)>=cuf.Halfop then return 1
  end
return 0
